// flirt_b.gs
//
// (C) 2021 p-dehnert

include "locomotive.gs"

class TGV_Duplex_Loc isclass Locomotive
{
    define int CYCLE_TIME = 3;
    int oldPantState, pantState = -1;
    bool oldDirection, direction = true;
    bool acsystem = true, systemchange = false;
    Train currentTrain;

    void SetPantograph(void)
    {
	if (acsystem)
	{
	    SetMeshAnimationState("panto1", false);
	    if (systemchange)
		SetMeshAnimationState("panto0", false);
	    else
		switch (pantState)
		{
		    case 1:
		    {
			SetMeshAnimationState("panto0", !direction);
			break;
		    }
		    case 2:
		    {
			SetMeshAnimationState("panto0", direction);
			break;
		    }
		    case 3:
		    {
			SetMeshAnimationState("panto0", true);
			break;
		    }
		    default:
			SetMeshAnimationState("panto0", false);
		}
	}
	else
	{
	    SetMeshAnimationState("panto0", false);
	    if (systemchange)
		SetMeshAnimationState("panto1", false);
	    else
		switch (pantState)
		{
		    case 1:
		    case 2:
		    case 3:
		    {
			SetMeshAnimationState("panto1", true);
			break;
		    }
		    default:
			SetMeshAnimationState("panto1", false);
		}
	}
    }
    

    public Soup GetProperties(void)
    {
	Soup soup = inherited();
	soup.SetNamedTag("pantstate", pantState);
	soup.SetNamedTag("direction", direction);
	soup.SetNamedTag("systemchange", systemchange);
	soup.SetNamedTag("acsystem", acsystem);
	return soup;
    }


    public void SetProperties(Soup soup)
    {
	inherited(soup);
	pantState = soup.GetNamedTagAsInt("pantState", -1);
	direction = soup.GetNamedTagAsBool("direction", true);
	systemchange = soup.GetNamedTagAsBool("systemchange", false);
	acsystem = soup.GetNamedTagAsBool("acsystem", true);
	SetPantograph();
    }

    void Handler206816(Message msg)
    {
	if (msg == null or msg.dst != me)
	    return;
	
	string[] tok = Str.Tokens(msg.minor, "=");
	if (tok.size() < 1)
	    return;
	
	if (tok[0] == "System" and tok.size() == 2)
	{
	    if ((tok[1] == "AC") or (tok[1] == "DC"))
		acsystem = (tok[1] == "AC");
	    else if ((tok[1] == "ON") or (tok[1] == "OFF"))
		systemchange = (tok[1] == "OFF");
	    SetPantograph();
	}
    }

    void VehicleHandler(Message msg)
    {
	if ((msg == null) or (msg.dst != me) and (msg.dst != null))
	    return;
	if ((msg.minor == "Coupled") or  (msg.minor == "Decoupled"))
	{
	    Train  myTrain = me.GetMyTrain();
	    if (myTrain)
	    {
		Vehicle[] vehicles = myTrain.GetVehicles();
		if ((me == vehicles[0]) or (me == vehicles[vehicles.size() - 1]))
		    SetMeshAnimationState("nezattel", false);
		else
		    SetMeshAnimationState("nezattel", true);
	    }
	}
    }
    
    thread void MonitorTrain(void)
    {
	Train newTrain = me.GetMyTrain();
	while (true)
	{
	    bool changed = (currentTrain != newTrain);
	    currentTrain = newTrain;
	    oldPantState = pantState;
	    pantState = currentTrain.GetPantographState();
	    changed = (changed or (oldPantState != pantState));
	    oldDirection = direction;
	    direction = me.GetDirectionRelativeToTrain();
	    changed = (changed or (oldDirection != direction));
	    if (changed)
		SetPantograph();
	    Sleep(CYCLE_TIME);
	    newTrain = me.GetMyTrain();
	}
    }
	
    void Init(void)
    {
	inherited();

	AddHandler(me, "Msg206816", "", "Handler206816");
	AddHandler(me, "Vehicle", "", "VehicleHandler");
	currentTrain = me.GetMyTrain();
	MonitorTrain();
    }

};